#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# pushbulletapi.py
#
# This file is part of PushBullet-Indicator
#
# Copyright (C) 2014
# Lorenzo Carbonell Cerezo <lorenzo.carbonell.cerezo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import requests
import mimetypes
from urllib.parse import quote
import os
from requests.auth import HTTPBasicAuth
from commons_configurator import Configuration
from commons_comun import internet_on
import platform

HOST = "https://api.pushbullet.com/v2"

def guess_type(filename):
	return mimetypes.guess_type(filename)[0] or 'application/octet-stream'

class PushBullet():
	def __init__(self):
		configuration = Configuration()
		self.oauth_access_token = configuration.get('oauth_access_token')
		self.source_user_iden = configuration.get('source_user_iden')
		self.source_device_iden = configuration.get('source_device_iden')
		self.contacts = {}
		self.devices = {}
		self.login = False
		if internet_on():
			user = self.getUser()
			print(user)	
			self.login = (user is not None)
			print(self.login)
			if self.login:
				self.source_user_iden = user['iden']
				self.updateContacts()
				self.updateDevices()
				if self.source_device_iden is None or len(self.source_device_iden)==0 or self.source_device_iden not in self.devices.keys():
					found = False
					model = platform.platform()	
					nickname = platform.linux_distribution()[0]
					print(nickname,model)
					for aiden in self.devices.keys():				
						if 'model' in self.devices[aiden].keys() and self.devices[aiden]['model'] == model and self.devices[aiden]['active'] == True:
							self.source_device_iden = aiden
							print('Device exists, I found it')
							found = True
					if found == False:
						print('Device doesnt exist, I will create new')
						ans = self.createDevice(nickname,model)
						self.source_device_iden = ans['iden']
						print('Device created')
				configuration.set('source_user_iden',self.source_user_iden)
				configuration.set('source_device_iden',self.source_device_iden)
				configuration.save()			

	def _request(self, method, url, postdata=None, params=None, files=None):
		if internet_on():
			headers = {"Accept": "application/json",
					   "Content-Type": "application/json",
					   "User-Agent": "pyPushBullet"}
			print(url)
			print('self.oauth_access_token',self.oauth_access_token)
			if postdata:
				postdata = json.dumps(postdata)
			try:
				r = requests.request(method,
									 url,
									 data=postdata,
									 params=params,
									 headers=headers,
									 files=files,
									 auth=HTTPBasicAuth(self.oauth_access_token, ""))

				r.raise_for_status()
				return r.json()
			except requests.exceptions.HTTPError as e:
				print(e)
				if e.response.status_code == 401:
					self.login = False
		return None
			
	def isLogin(self):
		return self.login
	
	def get_oauth_access_token(self):
		return self.oauth_access_token

	def set_oauth_access_token(self,oauth_access_token):
		self.oauth_access_token = oauth_access_token

	def get_source_user_iden(self):
		return self.source_user_iden

	def get_source_device_iden(self):
		return self.source_device_iden

	def updateDevices(self):
		""" Get devices
			https://docs.pushbullet.com/v2/devices

			Get a list of devices, and data about them.
		"""
		ans = {}
		devices = self._request("GET", HOST + "/devices")["devices"]
		for adevice in devices:
			ans[adevice['iden']] = adevice
		self.devices = ans

	def getDevices(self):
		return self.devices
		
	def getActiveDevices(self):
		""" Get active devices
			https://docs.pushbullet.com/v2/devices

			Get a list of active devices, and data about them.
		"""
		ans = {}
		for akey in self.devices.keys():
			adevice = self.devices[akey]
			if 'active' in adevice.keys() and adevice['active']:
				ans[akey] = adevice
		return ans

	def getPushableDevices(self):
		""" Get pushable devices
			https://docs.pushbullet.com/v2/devices

			Get a list of pushable devices, and data about them.
		"""
		ans = {}
		for akey in self.devices.keys():
			adevice = self.devices[akey]
			if 'pushable' in adevice.keys() and adevice['pushable']:
				ans[akey] = adevice
		return ans

	def createDevice(self, nickname, model):
		""" Create a new device
			https://docs.pushbullet.com/v2/devices

			Arguments:
			device_iden -- iden of device to push to
			nickname -- nickname of device to push to
		"""
		data = {"nickname": nickname,
				"type": "stream",
				"model": model}
		return self._request("POST", HOST + "/devices", data)


	def deleteDevice(self, device_iden):
		""" Delete a device
			https://docs.pushbullet.com/v2/devices

			Arguments:
			device_iden -- iden of device to push to
		"""

		return self._request("DELETE", HOST + "/devices/" + device_iden)

	def pushNote(self, identification, title, body, iscontact = False):
		""" Push a note
			https://docs.pushbullet.com/v2/pushes

			Arguments:
			device_iden -- iden of device to push to
			title -- a title for the note
			body -- the body of the note
		"""
		if iscontact:
			identificator = 'email'
		else:
			identificator = 'device_iden'		
		data = {"type": "note",
				identificator: identification,
				"title": title,
				"body": body}
		return self._request("POST", HOST + "/pushes", data)

	def sendReply(self, package_name, source_user_iden, target_device_iden, conversation_iden, message):
		""" Send a reply
			https://docs.pushbullet.com/v2/pushes

			Arguments:
			package_name -- package name
			source_user_iden -- iden of device to push from
			target_device_iden -- iden of device to push to
			conversation_iden -- iden of the conversation
			message -- The message
		"""
		data = {"type": "push",
				"push": {
					"type": "messaging_extension_reply",
					"package_name": package_name,
					"source_user_iden": source_user_iden,
					"target_device_iden":target_device_iden,
					"conversation_iden":conversation_iden,
					"message":message
					}
				}
		return self._request("POST", HOST + "/ephemerals", data)
		
	def pushCopy(self, body):
		""" Push a copy
			https://docs.pushbullet.com/v2/pushes

			Arguments:
			device_iden -- iden of device to push to
			title -- a title for the note
			body -- the body of the note
		"""
		#"source_device_iden":source_device_iden
		data = {"type": "push",
				"push": {
					"type": "clip",
					"body": body, 
					"source_user_iden": self.source_user_iden,
					"source_device_iden":self.source_device_iden
					}
				}
		return self._request("POST", HOST + "/ephemerals", data)

	def pushLink(self, identification, title, body, url, iscontact = False):
		""" Push a link
			https://docs.pushbullet.com/v2/pushes

			Arguments:
			device_iden -- iden of device to push to
			title -- link title
			url -- link url
		"""
		if iscontact:
			identificator = 'email'
		else:
			identificator = 'device_iden'		
		data = {"type": "link",
				identificator: identification,
				"title": title,
				"body":body,
				"url": url}
		return self._request("POST", HOST + "/pushes", data)

	def pushSMS(self, device_iden, number, message):
		""" Push a sms
			https://docs.pushbullet.com/v2/pushes

			Arguments:
			device_iden -- iden of device to push from
			number -- number of mobile to push to
			message -- the message
		"""
		data = {"type": "push",
				"push": {
					"type": "messaging_extension_reply",
					"package_name": "com.pushbullet.android",
					"source_user_iden": self.source_user_iden,
					"target_device_iden": device_iden,
					"conversation_iden": number,
					"message": message
				}
        }
		return self._request("POST", HOST + "/ephemerals", data)

	def pushFile(self, identification, afile, body, iscontact = False):
		""" Push a file
			https://docs.pushbullet.com/v2/pushes
			https://docs.pushbullet.com/v2/upload-request

			Arguments:
			device_iden -- iden of device to push to
			file_name -- name of the file
			file -- a file object
			file_type -- file mimetype, if not set, python-magic will be used
		"""
		if iscontact:
			identificator = 'email'
		else:
			identificator = 'device_iden'		
		file_name = quote(os.path.basename(afile))
		file_type = guess_type(afile)
		data = {"file_name": file_name,
				"file_type": file_type}

		upload_request = self._request("GET",
									   HOST + "/upload-request",
									   None,
									   data)

		upload = requests.post(upload_request["upload_url"],
							   data=upload_request["data"],
							   files={"file": open(afile,'rb')},
							   headers={"User-Agent": "pyPushBullet"})

		upload.raise_for_status()

		data = {"type": "file",
				identificator: identification,
				"file_name": file_name,
				"file_type": file_type,
				"file_url": upload_request["file_url"],
				"body": body}

		return self._request("POST", HOST + "/pushes", data)

	def getPushHistory(self, modified_after=0, cursor=None):
		""" Get Push History
			https://docs.pushbullet.com/v2/pushes

			Returns a list of pushes

			Arguments:
			modified_after -- Request pushes modified after this timestamp
			cursor -- Request another page of pushes (if necessary)
		"""
		data = {"modified_after": modified_after}
		if cursor:
			data["cursor"] = cursor
		return self._request("GET", HOST + "/pushes", None, data)["pushes"]
		
	def getLastPush(self):
		""" Get Push History
			https://docs.pushbullet.com/v2/pushes

			Returns a list of pushes

			Arguments:
			modified_after -- Request pushes modified after this timestamp
			cursor -- Request another page of pushes (if necessary)
		"""
		ans = self._request("GET", HOST + "/pushes", None)["pushes"]
		if len(ans)>0:
			return ans[0]
		return None

	def deletePush(self, push_iden):
		""" Delete push
			https://docs.pushbullet.com/v2/pushes

			Arguments:
			push_iden -- the iden of the push to delete
		"""
		return self._request("DELETE", HOST + "/pushes/" + push_iden)
	
	def updateContacts(self):
		""" Gets your contacts
			https://docs.pushbullet.com/v2/contacts

			returns a list of contacts
		"""
		ans = {}
		contacts = self._request("GET", HOST + "/contacts")["contacts"]
		for acontact in contacts:
			ans[acontact['iden']] = acontact
		self.contacts = ans
	
	def getContacts(self):
		return self.contacts
	
	def getPushableContacts(self):
		""" Gets your pushable contacts
			https://docs.pushbullet.com/v2/contacts

			returns a list of pushable contacts
		"""
		ans = {}
		for akey in self.contacts.keys():
			acontact = self.contacts[akey]
			if acontact['active'] and 'name' in acontact.keys() and 'email' in acontact.keys():
				ans[akey] = acontact
		return ans

	def getSubscriptions(self):
		""" List subscriptions
			https://docs.pushbullet.com/v2/subscriptions

			Arguments:
		"""
		return self._request("GET", HOST + "/subscriptions")
		
	def subscribeToChannel(self, channel_tag):
		""" Subscribe to channels to receive any updates pushed that channel
			https://docs.pushbullet.com/v2/subscriptions

			Arguments:
			channel_tag - The tag of the channel you wish to subscribe to.
		"""
		data = {"channel_tag": channel_tag
			}
				
		return self._request("POST", HOST + "/subscriptions", data)

	def unsubscribeFromAChannel(self, channel_iden):
		""" unubscribe from a channel
			https://docs.pushbullet.com/v2/subscriptions

			Arguments:
			channel_tag - The tag of the channel you wish to subscribe to.
		"""
		return self._request("DELETE", HOST + "/subscriptions/%s"%(channel_iden))

	def getInfoAboutChannel(self, channel_tag):
		""" Get info about channel
			https://docs.pushbullet.com/v2/subscriptions

			Arguments:
			channel_tag - The tag of the channel you wish to subscribe to.
		"""
		params = {"tag": channel_tag
			}
				
		return self._request("GET", HOST + "/channel-info", params=params)


	def createContact(self, email, name):
		""" Create a new contact
			https://docs.pushbullet.com/v2/contacts

			Arguments:
			name -- the name of the contact to push to
			email -- the email of the contact to push to
		"""
		data = {"name": name,
				"email": email}
				
		return self._request("POST", HOST + "/contacts", data)
		
	def deleteContact(self, contact_iden):
		""" Delete a contact
			https://docs.pushbullet.com/v2/contacts

			Arguments:
			contact_iden -- the iden of the contact to delete
		"""
		return self._request("DELETE", HOST + "/contacts/" + contact_iden)

	def getUser(self):
		""" Get this users information
			https://docs.pushbullet.com/v2/users
		"""
		return self._request("GET", HOST + "/users/me")

if __name__ == '__main__':
	from logindialog import LoginDialog
	pb = PushBullet()
	while(pb.isLogin() == False):
		ld = LoginDialog()
		ld.run()
		oauth_access_token = ld.code
		print(ld.code)
		configuration = Configuration()
		configuration.set('oauth_access_token',oauth_access_token)
		configuration.save()
		pb = PushBullet()
	print(pb.getActiveDevices())
	#print(pb.getDevices())
	#print(pb.getContacts())
	'''
	from commons_configurator import Configuration
	from logindialog import LoginDialog
	configuration = Configuration()
	oauth_access_token = configuration.get('oauth_access_token')
	while(oauth_access_token is None or len(oauth_access_token)==0):
		ld = LoginDialog()
		ld.run()
		oauth_access_token = ld.code
		print(oauth_access_token)
		configuration.set('oauth_access_token',oauth_access_token)
		configuration.save()
		configuration.read()
		oauth_access_token = configuration.get('oauth_access_token')		
	pb = PushBullet(oauth_access_token)
	print(pb.getDevices())
	print('-----------------------------------------')
	#print(pb.getContacts())
	#print(pb.getActiveDevices())
	#print(pb.getPushHistory())
	#print(pb.getPushableDevices())
	#print(pb.createDevice("test123"))	
	import platform
	print(platform.architecture())
	print(platform.machine())
	print(platform.node())
	print(platform.platform())
	print(platform.processor())
	print(platform.release())
	print(platform.system())
	print(platform.version())
	print(platform.uname())
	#pb.pushList('ujEzQo9UWJMdjzZ2K35GjQ','Ejemplo',['1','2','3','4'])
	#pb.pushNote('lorenzo.carbonell.cerezo@gmail.com','Ejemplo','Ejemplo',True)
	print('--------------')
	print(pb.getLastPush())
	print('--------------')
	print(pb.getUser())
	print('--------------')
	#print(pb.getSubscriptions())
	#print(pb.pushCopy('test'))
	#iden = pb.getInfoAboutChannel('jblow')['iden']
	#print('--------------')
	#print(iden)
	#print(pb.unsubscribeFromAChannel(iden))
	#print(pb.pushCopy('ujEzQo9UWJM','ujEzQo9UWJMsjAdnpXG7P2','Probando, probando, ...'))
	conversation_iden = {"package_name":"com.whatsapp","tag":None,"id":1}
	
	#conversation_iden = "+34600408294"
	#package_name = "com.pushbullet.android"
	#package_name = "com.whatsapp"
	'''
	#package_name = "org.telegram.messenger"
	#conversation_iden = {"package_name":"org.telegram.messenger","tag":None,"id":1000}
	#print(pb.sendReply(package_name,'ujEzQo9UWJM','ujEzQo9UWJMsjz56puQSIe',conversation_iden,"%s"%1551))
	print(pb.pushSMS('ujEzQo9UWJMsjz56puQSIe','+34600408294','Esto es una prueba desde el Motorola'))
	exit(0)

